@@ -78,7 +78,17 @@ def _is_builtin(node):
 
 
 _ACCEPTS_ITERATOR = {'iter', 'list', 'tuple', 'sorted', 'set', 'sum', 'any',
-                     'all', 'enumerate', 'dict', 'filter', 'reversed'}
+                     'all', 'enumerate', 'dict', 'filter', 'reversed',
+                     'max', 'min', 'frozenset'}
+ATTRIBUTES_ACCEPTS_ITERATOR = {'join', 'from_iterable'}
+_BUILTIN_METHOD_ACCEPTS_ITERATOR = {
+    'builtins.list.extend',
+    'builtins.dict.update',
+    'builtins.set.update',
+    '__builtin__.list.extend',
+    '__builtin__.dict.update',
+    '__builtin__.set.update',
+}
 DICT_METHODS = {'items', 'keys', 'values'}
 
 
@@ -106,14 +116,27 @@ def _in_iterating_context(node):
             if _is_builtin(parent_scope) and parent.func.name in _ACCEPTS_ITERATOR:
                 return True
         elif isinstance(parent.func, astroid.Attribute):
-            if parent.func.attrname == 'join':
+            if parent.func.attrname in ATTRIBUTES_ACCEPTS_ITERATOR:
                 return True
+            try:
+                inferred = next(parent.func.infer())
+            except astroid.InferenceError:
+                pass
+            else:
+                if inferred is not astroid.Uninferable:
+                    if inferred.qname() in _BUILTIN_METHOD_ACCEPTS_ITERATOR:
+                        return True
     # If the call is in an unpacking, there's no need to warn,
     # since it can be considered iterating.
     elif (isinstance(parent, astroid.Assign) and
           isinstance(parent.targets[0], (astroid.List, astroid.Tuple))):
         if len(parent.targets[0].elts) > 1:
             return True
+    # If the call is in a containment check, we consider that to
+    # be an iterating context
+    elif (isinstance(parent, astroid.Compare)
+          and len(parent.ops) == 1 and parent.ops[0][0] == 'in'):
+        return True
     return False
 
 
